# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SubNumberOrdersReport", "Filters"]


class Filters(BaseModel):
    """The filters that were applied to generate this report"""

    country_code: Optional[str] = None

    created_at_gt: Optional[datetime] = None

    created_at_lt: Optional[datetime] = None

    customer_reference: Optional[str] = None

    order_request_id: Optional[str] = None

    status: Optional[str] = None


class SubNumberOrdersReport(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    filters: Optional[Filters] = None
    """The filters that were applied to generate this report"""

    order_type: Optional[str] = None
    """The type of order report."""

    status: Optional[Literal["pending", "success", "failed", "expired"]] = None
    """Indicates the completion level of the sub number orders report.

    The report must have a status of 'success' before it can be downloaded.
    """

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""

    user_id: Optional[str] = None
    """The ID of the user who created the report."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from .telnyx_transcription_language import TelnyxTranscriptionLanguage

__all__ = ["TranscriptionEngineBConfigParam"]


class TranscriptionEngineBConfigParam(TypedDict, total=False):
    language: TelnyxTranscriptionLanguage
    """Language to use for speech recognition"""

    transcription_engine: Literal["B"]
    """Engine identifier for Telnyx transcription service"""

    transcription_model: Literal["openai/whisper-tiny", "openai/whisper-large-v3-turbo"]
    """The model to use for transcription."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..number_reservation import NumberReservation

__all__ = ["ActionExtendResponse"]


class ActionExtendResponse(BaseModel):
    data: Optional[NumberReservation] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.integrations.connection_list_response import ConnectionListResponse
from ....types.ai.integrations.connection_retrieve_response import ConnectionRetrieveResponse

__all__ = ["ConnectionsResource", "AsyncConnectionsResource"]


class ConnectionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ConnectionsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        user_connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionRetrieveResponse:
        """
        Get user setup integrations

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_connection_id:
            raise ValueError(f"Expected a non-empty value for `user_connection_id` but received {user_connection_id!r}")
        return self._get(
            f"/ai/integrations/connections/{user_connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionListResponse:
        """List user setup integrations"""
        return self._get(
            "/ai/integrations/connections",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionListResponse,
        )

    def delete(
        self,
        user_connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a specific integration connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_connection_id:
            raise ValueError(f"Expected a non-empty value for `user_connection_id` but received {user_connection_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/integrations/connections/{user_connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncConnectionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncConnectionsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        user_connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionRetrieveResponse:
        """
        Get user setup integrations

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_connection_id:
            raise ValueError(f"Expected a non-empty value for `user_connection_id` but received {user_connection_id!r}")
        return await self._get(
            f"/ai/integrations/connections/{user_connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionListResponse:
        """List user setup integrations"""
        return await self._get(
            "/ai/integrations/connections",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionListResponse,
        )

    async def delete(
        self,
        user_connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a specific integration connection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_connection_id:
            raise ValueError(f"Expected a non-empty value for `user_connection_id` but received {user_connection_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/integrations/connections/{user_connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ConnectionsResourceWithRawResponse:
    def __init__(self, connections: ConnectionsResource) -> None:
        self._connections = connections

        self.retrieve = to_raw_response_wrapper(
            connections.retrieve,
        )
        self.list = to_raw_response_wrapper(
            connections.list,
        )
        self.delete = to_raw_response_wrapper(
            connections.delete,
        )


class AsyncConnectionsResourceWithRawResponse:
    def __init__(self, connections: AsyncConnectionsResource) -> None:
        self._connections = connections

        self.retrieve = async_to_raw_response_wrapper(
            connections.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            connections.list,
        )
        self.delete = async_to_raw_response_wrapper(
            connections.delete,
        )


class ConnectionsResourceWithStreamingResponse:
    def __init__(self, connections: ConnectionsResource) -> None:
        self._connections = connections

        self.retrieve = to_streamed_response_wrapper(
            connections.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            connections.list,
        )
        self.delete = to_streamed_response_wrapper(
            connections.delete,
        )


class AsyncConnectionsResourceWithStreamingResponse:
    def __init__(self, connections: AsyncConnectionsResource) -> None:
        self._connections = connections

        self.retrieve = async_to_streamed_response_wrapper(
            connections.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            connections.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            connections.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["StreamBidirectionalSamplingRate"]

StreamBidirectionalSamplingRate: TypeAlias = Literal[8000, 16000, 22050, 24000, 48000]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    PushCredential,
    PushCredentialResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMobilePushCredentials:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        mobile_push_credential = client.mobile_push_credentials.create(
            create_mobile_push_credential_request={
                "alias": "LucyIosCredential",
                "certificate": "-----BEGIN CERTIFICATE----- MIIGVDCCBTKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END CERTIFICATE-----",
                "private_key": "-----BEGIN RSA PRIVATE KEY----- MIIEpQIBAAKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END RSA PRIVATE KEY-----",
                "type": "ios",
            },
        )
        assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.mobile_push_credentials.with_raw_response.create(
            create_mobile_push_credential_request={
                "alias": "LucyIosCredential",
                "certificate": "-----BEGIN CERTIFICATE----- MIIGVDCCBTKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END CERTIFICATE-----",
                "private_key": "-----BEGIN RSA PRIVATE KEY----- MIIEpQIBAAKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END RSA PRIVATE KEY-----",
                "type": "ios",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mobile_push_credential = response.parse()
        assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.mobile_push_credentials.with_streaming_response.create(
            create_mobile_push_credential_request={
                "alias": "LucyIosCredential",
                "certificate": "-----BEGIN CERTIFICATE----- MIIGVDCCBTKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END CERTIFICATE-----",
                "private_key": "-----BEGIN RSA PRIVATE KEY----- MIIEpQIBAAKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END RSA PRIVATE KEY-----",
                "type": "ios",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mobile_push_credential = response.parse()
            assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        mobile_push_credential = client.mobile_push_credentials.retrieve(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        )
        assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.mobile_push_credentials.with_raw_response.retrieve(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mobile_push_credential = response.parse()
        assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.mobile_push_credentials.with_streaming_response.retrieve(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mobile_push_credential = response.parse()
            assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `push_credential_id` but received ''"):
            client.mobile_push_credentials.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        mobile_push_credential = client.mobile_push_credentials.list()
        assert_matches_type(SyncDefaultFlatPagination[PushCredential], mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        mobile_push_credential = client.mobile_push_credentials.list(
            filter={
                "alias": "LucyCredential",
                "type": "ios",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[PushCredential], mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.mobile_push_credentials.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mobile_push_credential = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[PushCredential], mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.mobile_push_credentials.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mobile_push_credential = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[PushCredential], mobile_push_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        mobile_push_credential = client.mobile_push_credentials.delete(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        )
        assert mobile_push_credential is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.mobile_push_credentials.with_raw_response.delete(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mobile_push_credential = response.parse()
        assert mobile_push_credential is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.mobile_push_credentials.with_streaming_response.delete(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mobile_push_credential = response.parse()
            assert mobile_push_credential is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `push_credential_id` but received ''"):
            client.mobile_push_credentials.with_raw_response.delete(
                "",
            )


class TestAsyncMobilePushCredentials:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        mobile_push_credential = await async_client.mobile_push_credentials.create(
            create_mobile_push_credential_request={
                "alias": "LucyIosCredential",
                "certificate": "-----BEGIN CERTIFICATE----- MIIGVDCCBTKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END CERTIFICATE-----",
                "private_key": "-----BEGIN RSA PRIVATE KEY----- MIIEpQIBAAKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END RSA PRIVATE KEY-----",
                "type": "ios",
            },
        )
        assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.mobile_push_credentials.with_raw_response.create(
            create_mobile_push_credential_request={
                "alias": "LucyIosCredential",
                "certificate": "-----BEGIN CERTIFICATE----- MIIGVDCCBTKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END CERTIFICATE-----",
                "private_key": "-----BEGIN RSA PRIVATE KEY----- MIIEpQIBAAKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END RSA PRIVATE KEY-----",
                "type": "ios",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mobile_push_credential = await response.parse()
        assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.mobile_push_credentials.with_streaming_response.create(
            create_mobile_push_credential_request={
                "alias": "LucyIosCredential",
                "certificate": "-----BEGIN CERTIFICATE----- MIIGVDCCBTKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END CERTIFICATE-----",
                "private_key": "-----BEGIN RSA PRIVATE KEY----- MIIEpQIBAAKCAQEAsNlRJVZn9ZvXcECQm65czs... -----END RSA PRIVATE KEY-----",
                "type": "ios",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mobile_push_credential = await response.parse()
            assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        mobile_push_credential = await async_client.mobile_push_credentials.retrieve(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        )
        assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.mobile_push_credentials.with_raw_response.retrieve(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mobile_push_credential = await response.parse()
        assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.mobile_push_credentials.with_streaming_response.retrieve(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mobile_push_credential = await response.parse()
            assert_matches_type(PushCredentialResponse, mobile_push_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `push_credential_id` but received ''"):
            await async_client.mobile_push_credentials.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        mobile_push_credential = await async_client.mobile_push_credentials.list()
        assert_matches_type(AsyncDefaultFlatPagination[PushCredential], mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        mobile_push_credential = await async_client.mobile_push_credentials.list(
            filter={
                "alias": "LucyCredential",
                "type": "ios",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[PushCredential], mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.mobile_push_credentials.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mobile_push_credential = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[PushCredential], mobile_push_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.mobile_push_credentials.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mobile_push_credential = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[PushCredential], mobile_push_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        mobile_push_credential = await async_client.mobile_push_credentials.delete(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        )
        assert mobile_push_credential is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.mobile_push_credentials.with_raw_response.delete(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mobile_push_credential = await response.parse()
        assert mobile_push_credential is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.mobile_push_credentials.with_streaming_response.delete(
            "0ccc7b76-4df3-4bca-a05a-3da1ecc389f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mobile_push_credential = await response.parse()
            assert mobile_push_credential is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `push_credential_id` but received ''"):
            await async_client.mobile_push_credentials.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict
from typing_extensions import TypeAlias

__all__ = ["CampaignGetOperationStatusResponse"]

CampaignGetOperationStatusResponse: TypeAlias = Dict[str, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .auto_resp_config import AutoRespConfig
from ..shared.messaging_pagination_meta import MessagingPaginationMeta

__all__ = ["AutorespConfigListResponse"]


class AutorespConfigListResponse(BaseModel):
    """List of Auto-Response Settings"""

    data: List[AutoRespConfig]

    meta: MessagingPaginationMeta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["DynamicEmergencyAddressListParams", "Filter"]


class DynamicEmergencyAddressListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[status], filter[country_code]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[status], filter[country_code]
    """

    country_code: str
    """Filter by country code."""

    status: Literal["pending", "activated", "rejected"]
    """Filter by status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AlphanumericSenderIDCreateParams"]


class AlphanumericSenderIDCreateParams(TypedDict, total=False):
    alphanumeric_sender_id: Required[str]
    """The alphanumeric sender ID string."""

    messaging_profile_id: Required[str]
    """The messaging profile to associate the sender ID with."""

    us_long_code_fallback: str
    """A US long code number to use as fallback when sending to US destinations."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._compat import cached_property
from .connections import (
    ConnectionsResource,
    AsyncConnectionsResource,
    ConnectionsResourceWithRawResponse,
    AsyncConnectionsResourceWithRawResponse,
    ConnectionsResourceWithStreamingResponse,
    AsyncConnectionsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.integration_list_response import IntegrationListResponse
from ....types.ai.integration_retrieve_response import IntegrationRetrieveResponse

__all__ = ["IntegrationsResource", "AsyncIntegrationsResource"]


class IntegrationsResource(SyncAPIResource):
    @cached_property
    def connections(self) -> ConnectionsResource:
        return ConnectionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> IntegrationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return IntegrationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IntegrationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return IntegrationsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        integration_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationRetrieveResponse:
        """
        Retrieve integration details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not integration_id:
            raise ValueError(f"Expected a non-empty value for `integration_id` but received {integration_id!r}")
        return self._get(
            f"/ai/integrations/{integration_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationListResponse:
        """List all available integrations."""
        return self._get(
            "/ai/integrations",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationListResponse,
        )


class AsyncIntegrationsResource(AsyncAPIResource):
    @cached_property
    def connections(self) -> AsyncConnectionsResource:
        return AsyncConnectionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncIntegrationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIntegrationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIntegrationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncIntegrationsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        integration_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationRetrieveResponse:
        """
        Retrieve integration details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not integration_id:
            raise ValueError(f"Expected a non-empty value for `integration_id` but received {integration_id!r}")
        return await self._get(
            f"/ai/integrations/{integration_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationListResponse:
        """List all available integrations."""
        return await self._get(
            "/ai/integrations",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationListResponse,
        )


class IntegrationsResourceWithRawResponse:
    def __init__(self, integrations: IntegrationsResource) -> None:
        self._integrations = integrations

        self.retrieve = to_raw_response_wrapper(
            integrations.retrieve,
        )
        self.list = to_raw_response_wrapper(
            integrations.list,
        )

    @cached_property
    def connections(self) -> ConnectionsResourceWithRawResponse:
        return ConnectionsResourceWithRawResponse(self._integrations.connections)


class AsyncIntegrationsResourceWithRawResponse:
    def __init__(self, integrations: AsyncIntegrationsResource) -> None:
        self._integrations = integrations

        self.retrieve = async_to_raw_response_wrapper(
            integrations.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            integrations.list,
        )

    @cached_property
    def connections(self) -> AsyncConnectionsResourceWithRawResponse:
        return AsyncConnectionsResourceWithRawResponse(self._integrations.connections)


class IntegrationsResourceWithStreamingResponse:
    def __init__(self, integrations: IntegrationsResource) -> None:
        self._integrations = integrations

        self.retrieve = to_streamed_response_wrapper(
            integrations.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            integrations.list,
        )

    @cached_property
    def connections(self) -> ConnectionsResourceWithStreamingResponse:
        return ConnectionsResourceWithStreamingResponse(self._integrations.connections)


class AsyncIntegrationsResourceWithStreamingResponse:
    def __init__(self, integrations: AsyncIntegrationsResource) -> None:
        self._integrations = integrations

        self.retrieve = async_to_streamed_response_wrapper(
            integrations.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            integrations.list,
        )

    @cached_property
    def connections(self) -> AsyncConnectionsResourceWithStreamingResponse:
        return AsyncConnectionsResourceWithStreamingResponse(self._integrations.connections)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["UserBundleCreateParams", "Item"]


class UserBundleCreateParams(TypedDict, total=False):
    idempotency_key: str
    """Idempotency key for the request.

    Can be any UUID, but should always be unique for each request.
    """

    items: Iterable[Item]

    authorization_bearer: str
    """Authenticates the request with your Telnyx API V2 KEY"""


class Item(TypedDict, total=False):
    billing_bundle_id: Required[str]
    """Quantity of user bundles to order."""

    quantity: Required[int]
    """Quantity of user bundles to order."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .test_status import TestStatus as TestStatus
from .run_list_params import RunListParams as RunListParams
from .test_run_response import TestRunResponse as TestRunResponse
from .run_trigger_params import RunTriggerParams as RunTriggerParams
from .test_run_detail_result import TestRunDetailResult as TestRunDetailResult
from .test_suite_list_response import TestSuiteListResponse as TestSuiteListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CallStreamsJsonParams"]


class CallStreamsJsonParams(TypedDict, total=False):
    account_sid: Required[str]

    bidirectional_codec: Annotated[Literal["PCMU", "PCMA", "G722"], PropertyInfo(alias="BidirectionalCodec")]
    """Indicates codec for bidirectional streaming RTP payloads.

    Used only with stream_bidirectional_mode=rtp. Case sensitive.
    """

    bidirectional_mode: Annotated[Literal["mp3", "rtp"], PropertyInfo(alias="BidirectionalMode")]
    """Configures method of bidirectional streaming (mp3, rtp)."""

    name: Annotated[str, PropertyInfo(alias="Name")]
    """The user specified name of Stream."""

    status_callback: Annotated[str, PropertyInfo(alias="StatusCallback")]
    """Url where status callbacks will be sent."""

    status_callback_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="StatusCallbackMethod")]
    """HTTP method used to send status callbacks."""

    track: Annotated[Literal["inbound_track", "outbound_track", "both_tracks"], PropertyInfo(alias="Track")]
    """Tracks to be included in the stream"""

    url: Annotated[str, PropertyInfo(alias="Url")]
    """The destination WebSocket address where the stream is going to be delivered."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.payment import auto_recharge_pref_update_params
from ...types.payment.auto_recharge_pref_list_response import AutoRechargePrefListResponse
from ...types.payment.auto_recharge_pref_update_response import AutoRechargePrefUpdateResponse

__all__ = ["AutoRechargePrefsResource", "AsyncAutoRechargePrefsResource"]


class AutoRechargePrefsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AutoRechargePrefsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AutoRechargePrefsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AutoRechargePrefsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AutoRechargePrefsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        enabled: bool | Omit = omit,
        invoice_enabled: bool | Omit = omit,
        preference: Literal["credit_paypal", "ach"] | Omit = omit,
        recharge_amount: str | Omit = omit,
        threshold_amount: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRechargePrefUpdateResponse:
        """
        Update payment auto recharge preferences.

        Args:
          enabled: Whether auto recharge is enabled.

          preference: The payment preference for auto recharge.

          recharge_amount: The amount to recharge the account, the actual recharge amount will be the
              amount necessary to reach the threshold amount plus the recharge amount.

          threshold_amount: The threshold amount at which the account will be recharged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            "/payment/auto_recharge_prefs",
            body=maybe_transform(
                {
                    "enabled": enabled,
                    "invoice_enabled": invoice_enabled,
                    "preference": preference,
                    "recharge_amount": recharge_amount,
                    "threshold_amount": threshold_amount,
                },
                auto_recharge_pref_update_params.AutoRechargePrefUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRechargePrefUpdateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRechargePrefListResponse:
        """Returns the payment auto recharge preferences."""
        return self._get(
            "/payment/auto_recharge_prefs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRechargePrefListResponse,
        )


class AsyncAutoRechargePrefsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAutoRechargePrefsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAutoRechargePrefsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAutoRechargePrefsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAutoRechargePrefsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        enabled: bool | Omit = omit,
        invoice_enabled: bool | Omit = omit,
        preference: Literal["credit_paypal", "ach"] | Omit = omit,
        recharge_amount: str | Omit = omit,
        threshold_amount: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRechargePrefUpdateResponse:
        """
        Update payment auto recharge preferences.

        Args:
          enabled: Whether auto recharge is enabled.

          preference: The payment preference for auto recharge.

          recharge_amount: The amount to recharge the account, the actual recharge amount will be the
              amount necessary to reach the threshold amount plus the recharge amount.

          threshold_amount: The threshold amount at which the account will be recharged.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            "/payment/auto_recharge_prefs",
            body=await async_maybe_transform(
                {
                    "enabled": enabled,
                    "invoice_enabled": invoice_enabled,
                    "preference": preference,
                    "recharge_amount": recharge_amount,
                    "threshold_amount": threshold_amount,
                },
                auto_recharge_pref_update_params.AutoRechargePrefUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRechargePrefUpdateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRechargePrefListResponse:
        """Returns the payment auto recharge preferences."""
        return await self._get(
            "/payment/auto_recharge_prefs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRechargePrefListResponse,
        )


class AutoRechargePrefsResourceWithRawResponse:
    def __init__(self, auto_recharge_prefs: AutoRechargePrefsResource) -> None:
        self._auto_recharge_prefs = auto_recharge_prefs

        self.update = to_raw_response_wrapper(
            auto_recharge_prefs.update,
        )
        self.list = to_raw_response_wrapper(
            auto_recharge_prefs.list,
        )


class AsyncAutoRechargePrefsResourceWithRawResponse:
    def __init__(self, auto_recharge_prefs: AsyncAutoRechargePrefsResource) -> None:
        self._auto_recharge_prefs = auto_recharge_prefs

        self.update = async_to_raw_response_wrapper(
            auto_recharge_prefs.update,
        )
        self.list = async_to_raw_response_wrapper(
            auto_recharge_prefs.list,
        )


class AutoRechargePrefsResourceWithStreamingResponse:
    def __init__(self, auto_recharge_prefs: AutoRechargePrefsResource) -> None:
        self._auto_recharge_prefs = auto_recharge_prefs

        self.update = to_streamed_response_wrapper(
            auto_recharge_prefs.update,
        )
        self.list = to_streamed_response_wrapper(
            auto_recharge_prefs.list,
        )


class AsyncAutoRechargePrefsResourceWithStreamingResponse:
    def __init__(self, auto_recharge_prefs: AsyncAutoRechargePrefsResource) -> None:
        self._auto_recharge_prefs = auto_recharge_prefs

        self.update = async_to_streamed_response_wrapper(
            auto_recharge_prefs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            auto_recharge_prefs.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.porting_order_status import PortingOrderStatus

__all__ = ["PortingEventStatusChangedEvent", "Payload"]


class Payload(BaseModel):
    """The webhook payload for the porting_order.status_changed event"""

    id: Optional[str] = None
    """Identifies the porting order that was moved."""

    customer_reference: Optional[str] = None
    """Identifies the customer reference associated with the porting order."""

    status: Optional[PortingOrderStatus] = None
    """Porting order status"""

    support_key: Optional[str] = None
    """Identifies the support key associated with the porting order."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the porting order was moved."""

    webhook_url: Optional[str] = None
    """The URL to send the webhook to."""


class PortingEventStatusChangedEvent(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook", "webhook_v1"]]] = None
    """Indicates the notification methods used."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    event_type: Optional[
        Literal[
            "porting_order.deleted",
            "porting_order.loa_updated",
            "porting_order.messaging_changed",
            "porting_order.status_changed",
            "porting_order.sharing_token_expired",
            "porting_order.new_comment",
            "porting_order.split",
        ]
    ] = None
    """Identifies the event type"""

    payload: Optional[Payload] = None
    """The webhook payload for the porting_order.status_changed event"""

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    porting_order_id: Optional[str] = None
    """Identifies the porting order associated with the event."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .phone_numbers_job import PhoneNumbersJob

__all__ = ["JobDeleteBatchResponse"]


class JobDeleteBatchResponse(BaseModel):
    data: Optional[PhoneNumbersJob] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["StreamStreamingSidJsonParams"]


class StreamStreamingSidJsonParams(TypedDict, total=False):
    account_sid: Required[str]

    call_sid: Required[str]

    status: Annotated[Literal["stopped"], PropertyInfo(alias="Status")]
    """The status of the Stream you wish to update."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .call_control_command_result import CallControlCommandResult

__all__ = ["ActionAddAIAssistantMessagesResponse"]


class ActionAddAIAssistantMessagesResponse(BaseModel):
    data: Optional[CallControlCommandResult] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ActionShareParams"]


class ActionShareParams(TypedDict, total=False):
    expires_in_seconds: int
    """The number of seconds the token will be valid for"""

    permissions: Literal["porting_order.document.read", "porting_order.document.update"]
    """The permissions the token will have"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["WireguardInterfaceCreateParams"]


class WireguardInterfaceCreateParams(TypedDict, total=False):
    region_code: Required[str]
    """The region the interface should be deployed to."""

    enable_sip_trunking: bool
    """Enable SIP traffic forwarding over VPN interface."""

    name: str
    """A user specified name for the interface."""

    network_id: str
    """The id of the network associated with the interface."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_fork_started import CallForkStarted

__all__ = ["CallForkStartedWebhookEvent"]


class CallForkStartedWebhookEvent(BaseModel):
    data: Optional[CallForkStarted] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ..._models import BaseModel

__all__ = ["APIError", "Source"]


class Source(BaseModel):
    parameter: Optional[str] = None
    """Indicates which query parameter caused the error."""

    pointer: Optional[str] = None
    """JSON pointer (RFC6901) to the offending entity."""


class APIError(BaseModel):
    code: str

    title: str

    description: Optional[str] = None

    meta: Optional[Dict[str, object]] = None

    source: Optional[Source] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AzureVoiceSettings"]


class AzureVoiceSettings(BaseModel):
    type: Literal["azure"]
    """Voice settings provider type"""

    api_key_ref: Optional[str] = None
    """
    The `identifier` for an integration secret that refers to your Azure Speech API
    key.
    """

    deployment_id: Optional[str] = None
    """The deployment ID for a custom Azure neural voice."""

    effect: Optional[Literal["eq_car", "eq_telecomhp8k"]] = None
    """Audio effect to apply."""

    gender: Optional[Literal["Male", "Female"]] = None
    """Voice gender filter."""

    region: Optional[str] = None
    """The Azure region for the Speech service (e.g., `eastus`, `westeurope`).

    Required when using a custom API key.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["SiprecConnectorRetrieveResponse", "Data"]


class Data(BaseModel):
    app_subdomain: Optional[str] = None
    """Subdomain to route calls when using Telnyx SRS (optional)."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date/time of creation."""

    host: Optional[str] = None
    """Hostname/IPv4 address of the SIPREC SRS."""

    name: Optional[str] = None
    """Name for the SIPREC connector resource."""

    port: Optional[int] = None
    """Port for the SIPREC SRS."""

    record_type: Optional[str] = None

    updated_at: Optional[str] = None
    """ISO 8601 formatted date/time of last update."""


class SiprecConnectorRetrieveResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import oauth_grant_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.oauth_grant import OAuthGrant
from ..types.oauth_grant_delete_response import OAuthGrantDeleteResponse
from ..types.oauth_grant_retrieve_response import OAuthGrantRetrieveResponse

__all__ = ["OAuthGrantsResource", "AsyncOAuthGrantsResource"]


class OAuthGrantsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OAuthGrantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return OAuthGrantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OAuthGrantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return OAuthGrantsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthGrantRetrieveResponse:
        """
        Retrieve a single OAuth grant by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/oauth_grants/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthGrantRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[OAuthGrant]:
        """
        Retrieve a paginated list of OAuth grants for the authenticated user

        Args:
          page_number: Page number

          page_size: Number of results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/oauth_grants",
            page=SyncDefaultFlatPagination[OAuthGrant],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    oauth_grant_list_params.OAuthGrantListParams,
                ),
            ),
            model=OAuthGrant,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthGrantDeleteResponse:
        """
        Revoke an OAuth grant

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/oauth_grants/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthGrantDeleteResponse,
        )


class AsyncOAuthGrantsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOAuthGrantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOAuthGrantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOAuthGrantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncOAuthGrantsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthGrantRetrieveResponse:
        """
        Retrieve a single OAuth grant by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/oauth_grants/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthGrantRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OAuthGrant, AsyncDefaultFlatPagination[OAuthGrant]]:
        """
        Retrieve a paginated list of OAuth grants for the authenticated user

        Args:
          page_number: Page number

          page_size: Number of results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/oauth_grants",
            page=AsyncDefaultFlatPagination[OAuthGrant],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    oauth_grant_list_params.OAuthGrantListParams,
                ),
            ),
            model=OAuthGrant,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthGrantDeleteResponse:
        """
        Revoke an OAuth grant

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/oauth_grants/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthGrantDeleteResponse,
        )


class OAuthGrantsResourceWithRawResponse:
    def __init__(self, oauth_grants: OAuthGrantsResource) -> None:
        self._oauth_grants = oauth_grants

        self.retrieve = to_raw_response_wrapper(
            oauth_grants.retrieve,
        )
        self.list = to_raw_response_wrapper(
            oauth_grants.list,
        )
        self.delete = to_raw_response_wrapper(
            oauth_grants.delete,
        )


class AsyncOAuthGrantsResourceWithRawResponse:
    def __init__(self, oauth_grants: AsyncOAuthGrantsResource) -> None:
        self._oauth_grants = oauth_grants

        self.retrieve = async_to_raw_response_wrapper(
            oauth_grants.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            oauth_grants.list,
        )
        self.delete = async_to_raw_response_wrapper(
            oauth_grants.delete,
        )


class OAuthGrantsResourceWithStreamingResponse:
    def __init__(self, oauth_grants: OAuthGrantsResource) -> None:
        self._oauth_grants = oauth_grants

        self.retrieve = to_streamed_response_wrapper(
            oauth_grants.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            oauth_grants.list,
        )
        self.delete = to_streamed_response_wrapper(
            oauth_grants.delete,
        )


class AsyncOAuthGrantsResourceWithStreamingResponse:
    def __init__(self, oauth_grants: AsyncOAuthGrantsResource) -> None:
        self._oauth_grants = oauth_grants

        self.retrieve = async_to_streamed_response_wrapper(
            oauth_grants.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            oauth_grants.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            oauth_grants.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["UsageReportRetrieveSpeechToTextParams"]


class UsageReportRetrieveSpeechToTextParams(TypedDict, total=False):
    end_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    start_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PaymentCreateStoredPaymentTransactionParams"]


class PaymentCreateStoredPaymentTransactionParams(TypedDict, total=False):
    amount: Required[str]
    """Amount in dollars and cents, e.g. "120.00" """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .fax_application import FaxApplication

__all__ = ["FaxApplicationRetrieveResponse"]


class FaxApplicationRetrieveResponse(BaseModel):
    data: Optional[FaxApplication] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ConferenceUpdateParams"]


class ConferenceUpdateParams(TypedDict, total=False):
    account_sid: Required[str]

    announce_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="AnnounceMethod")]
    """The HTTP method used to call the `AnnounceUrl`. Defaults to `POST`."""

    announce_url: Annotated[str, PropertyInfo(alias="AnnounceUrl")]
    """The URL we should call to announce something into the conference.

    The URL may return an MP3 file, a WAV file, or a TwiML document that contains
    `<Play>`, `<Say>`, `<Pause>`, or `<Redirect>` verbs.
    """

    status: Annotated[str, PropertyInfo(alias="Status")]
    """The new status of the resource.

    Specifying `completed` will end the conference and hang up all participants.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["VersionConfig"]


class VersionConfig(BaseModel):
    """Configuration for a single version in canary deploy."""

    percentage: float
    """Percentage of traffic for this version [1-99]"""

    version_id: str
    """Version ID string that references assistant_versions.version_id"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["AutorespConfigListParams", "CreatedAt", "UpdatedAt"]


class AutorespConfigListParams(TypedDict, total=False):
    country_code: str

    created_at: CreatedAt
    """Consolidated created_at parameter (deepObject style).

    Originally: created_at[gte], created_at[lte]
    """

    updated_at: UpdatedAt
    """Consolidated updated_at parameter (deepObject style).

    Originally: updated_at[gte], updated_at[lte]
    """


class CreatedAt(TypedDict, total=False):
    """Consolidated created_at parameter (deepObject style).

    Originally: created_at[gte], created_at[lte]
    """

    gte: str

    lte: str


class UpdatedAt(TypedDict, total=False):
    """Consolidated updated_at parameter (deepObject style).

    Originally: updated_at[gte], updated_at[lte]
    """

    gte: str

    lte: str
